unit HDSGrds;

//////////////////////////////////////////////////////////////////////////
//By Hades (Colombia)                                                   //
//virrato@yahoo.com                                                     //
//Freeware                                                              //
//////////////////////////////////////////////////////////////////////////

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, Extctrls, HDSFuncs, Math, FileCtrl, ShellApi, DBCtrls, Db;

type
  THDSGlyphTextLayout = (gtlTop, gtlBottom);

  THDSGlyphEvent = procedure(Sender: TObject; AIndex: integer;
                             const ATitle: string) of object;

  THDSCustomGlyphGrid = class;

  THDSGlyph = class(TCollectionItem)
  private
    FBitmap: TBitmap;
    FGlyphIndex: integer;

  public
    constructor Create(AOwner: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure AssignTo(Dest: TPersistent); override;
    property Bitmap: TBitmap read FBitmap write FBitmap;
    property GlyphIndex: integer read FGlyphIndex write FGlyphIndex;
  end;

  THDSGlyphs = class(TCollection)
  private
    FGrid: THDSCustomGlyphGrid;
    function GetItem(Index: integer): THDSGlyph;
    procedure SetItem(Index: integer; NewItem: THDSGlyph);

  protected
    function GetOwner: TPersistent; override;

  public
    constructor Create(AGrid: THDSCustomGlyphGrid;
                       ItemClass: TCollectionItemClass);
    function Add: THDSGlyph;
    function FindIndex(Index: integer): integer;
    procedure ClearIndexes;
    property Grid: THDSCustomGlyphGrid read FGrid;
    property Items[Index: integer]: THDSGlyph read GetItem
                                              write SetItem; default;
  end;

  THDSCustomGlyphGrid = class(TCustomGrid)
  private
    FCurrentGlyph: integer;
    FIsReading: boolean;
    FEllipseStyle: THDSEllipseStyle;
    FGlyphMargins: TMargins;
    FGlyphBorder: TBorderStyle;
    FGlyphHeight: integer;
    FGlyphTextLayout: THDSGlyphTextLayout;
    FGlyphWidth: integer;
    FPicture: TPicture;
    FItemsCount: integer;
    FLastCol: longint;
    FLastRow: longint;
    FShowTitles: boolean;
    FTextHeight: integer;
    FTitle: string;
    FLastGlyphs: THDSGlyphs;
    FOnDrawCell: TDrawCellEvent;
    FOnGlyphClick: THDSGlyphEvent;
    FOnGlyphDblClick: THDSGlyphEvent;
    FOnSelectCell: TSelectCellEvent;

    procedure DoMarginsChanged(Sender: TObject);
    function GetColGlyphCount: integer;
    function GetRowGlyphCount: integer;
    procedure SetEllipseStyle(const Value: THDSEllipseStyle);
    procedure SetGlyphBorder(const Value: TBorderStyle);
    procedure SetGlyphHeight(const Value: integer);
    procedure SetGlyphTextLayout(const Value: THDSGlyphTextLayout);
    procedure SetGlyphWidth(const Value: integer);
    function GetSelectedIndex: integer;
    procedure SetSelectedIndex(const Value: integer);
    procedure SetShowTitles(const Value: boolean);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;

  protected
    procedure Click; override;
    function CreatePicture: TPicture; dynamic;
    function CreateMargins: TMargins; dynamic;
    function CreateLastGlyphs: THDSGlyphs; dynamic;
    procedure DblClick; override;
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect;
                      AState: TGridDrawState); override;
    function GetCurrentCell: TGridCoord;
    function GetTitle(ACol, ARow: longint): string; dynamic; abstract;
    procedure GlyphClick(ACol, ARow: longint; const ATitle: string); dynamic;
    procedure GlyphDblClick(ACol, ARow: longint; const ATitle: string); dynamic;
    function SelectCell(ACol, ARow: Longint): boolean; override;
    function UpdateHint(ACol, ARow: integer): boolean; dynamic; abstract;
    function UpdateImage(ACol, ARow: integer): boolean; dynamic; abstract;
    function UpdateTitle(ACol, ARow: integer): boolean; dynamic; abstract;
    procedure WndProc(var Message: TMessage); override;

    property CurrentGlyph: integer read FCurrentGlyph write FCurrentGlyph;
    property LastGlyphs: THDSGlyphs read FLastGlyphs write FLastGlyphs;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GridCoordToIndex(ACol, ARow: longint): integer; overload;
    function GridCoordToIndex(GridCoord: TGridCoord): integer; overload;
    function IndexToGridCoord(AIndex: integer): TGridCoord;

    property Canvas;
    property Col;
    property ColGlyphCount: integer read GetColGlyphCount;
    property GridHeight;
    property GridWidth;
    property IsReading: boolean read FIsReading write FIsReading;
    property ItemsCount: integer read FItemsCount write FItemsCount;
    property Picture: TPicture read FPicture write FPicture;
    property LeftCol;
    property Row;
    property RowGlyphCount: integer read GetRowGlyphCount;
    property SelectedIndex: integer read GetSelectedIndex
                                    write SetSelectedIndex;
    property Title: string read FTitle write FTitle;
    property TopRow;

  published
    property Align;
    property Anchors;
    property BorderStyle;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property EllipseStyle: THDSEllipseStyle read FEllipseStyle
                            write SetEllipseStyle default ellEnd;
    property Enabled;
    property Font;
    property GlyphHeight: integer read FGlyphHeight write SetGlyphHeight
                                  default 64;
    property GlyphTextLayout: THDSGlyphTextLayout read FGlyphTextLayout
                                                write SetGlyphTextLayout
                                                default gtlTop;
    property GlyphWidth: integer read FGlyphWidth write SetGlyphWidth
                                 default 64;
    property GlyphMargins: TMargins read FGlyphMargins write FGlyphMargins;
    property GlyphBorder: TBorderStyle read FGlyphBorder write SetGlyphBorder
                                       default bsSingle;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ScrollBars;
    property ShowHint;
    property ShowTitles: boolean read FShowTitles write SetShowTitles
                                 default False;
    property TabOrder;
    property TabStop;
    property Visible;
    property VisibleColCount;
    property VisibleRowCount;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawCell: TDrawCellEvent read FOnDrawCell write FOnDrawCell;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGlyphClick: THDSGlyphEvent read FOnGlyphClick
                                          write FOnGlyphClick;
    property OnGlyphDblClick: THDSGlyphEvent read FOnGlyphDblClick
                                             write FOnGlyphDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnSelectCell: TSelectCellEvent read FOnSelectCell
                                            write FOnSelectCell;
    property OnStartDock;
    property OnStartDrag;
  end;

  THDSGlyphGrid = class(THDSCustomGlyphGrid)
  private
    FDirectory: string;
    FFilter: string;
    FFileList: TStringList;
    FSupportedExts: string;

    procedure SetDirectory(const Value: string);
    procedure SetFilter(const Value: string);

  protected
    function GetSupportedExts: string; dynamic;
    function GetTitle(ACol, ARow: longint): string; override;
    function UpdateFiles: boolean; dynamic;
    function UpdateHint(ACol, ARow: integer): boolean; override;
    function UpdateImage(ACol, ARow: integer): boolean; override;
    function UpdateTitle(ACol, ARow: integer): boolean; override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure RefreshDirectory; dynamic;
    property FileList: TStringList read FFileList;
    property SupportedExts: string read FSupportedExts;

  published
    property Directory: string read FDirectory write SetDirectory;
    property Filter: string read FFilter write SetFilter;
  end;

  THDSEllipseGrid = class(TStringGrid)
  private
    { Private declarations }
     FAlignment: TAlignment;
     FBevelWidth: TBevelWidth;
     FEllipseStyle: THDSEllipseStyle;

     procedure SetAlignment(const Value: TAlignment);
     procedure SetBevelWidth(const Value: TBevelWidth);
     procedure SetEllipseStyle(const Value: THDSEllipseStyle);

  protected
     { Protected declarations }
     procedure DrawCell(ACol,ARow: longint; ARect: TRect;
                       AState : TGridDrawState); override;

  public
     { Public declarations }
     constructor Create(AOwner: TComponent); override;

  published
    { Published declarations }
     property Alignment: TAlignment read FAlignment write SetAlignment
                         default taLeftJustify;
     property BevelWidth: TBevelWidth read FBevelWidth write SetBevelWidth
                                       default 1;
     property EllipseStyle: THDSEllipseStyle read FEllipseStyle
                             write SetEllipseStyle default ellEnd;
  end;

procedure Register;

implementation
//THDSGlyph
constructor THDSGlyph.Create(AOwner: TCollection);
begin
  inherited Create(AOwner);
  FGlyphIndex:= -1;
  FBitmap:= TBitmap.Create;
  FBitmap.PixelFormat:= pf32bit; //Maxima calidad
end;

destructor THDSGlyph.Destroy;
begin
  FBitmap.Free;
  FBitmap:= nil;
  inherited Destroy;
end;

procedure THDSGlyph.Assign(Source: TPersistent);
begin
  if Source is THDSGlyph then
    with THDSGlyph(Source) do
    begin
      Self.FGlyphIndex:= FGlyphIndex;
      Self.FBitmap.Assign(FBitmap);
    end
  else
    inherited Assign(Source);
end;

procedure THDSGlyph.AssignTo(Dest: TPersistent);
begin
  if Dest is THDSGlyph then
    with THDSGlyph(Dest) do
    begin
      FGlyphIndex:= Self.FGlyphIndex;
      FBitmap.Assign(Self.FBitmap);
    end
  else
    inherited AssignTo(Dest);
end;

//THDSGlyphs
constructor THDSGlyphs.Create(AGrid: THDSCustomGlyphGrid;
                              ItemClass: TCollectionItemClass);
begin
  inherited Create(ItemClass);
  FGrid:= AGrid;
end;

function THDSGlyphs.GetItem(Index: integer): THDSGlyph;
begin
  Result:= THDSGlyph(inherited GetItem(Index));
end;

procedure THDSGlyphs.SetItem(Index: integer; NewItem: THDSGlyph);
begin
  inherited SetItem(Index, NewItem);
end;

function THDSGlyphs.GetOwner: TPersistent;
begin
  Result:= FGrid;
end;

function THDSGlyphs.Add: THDSGlyph;
begin
  Result:= THDSGlyph(inherited Add);
end;

function THDSGlyphs.FindIndex(Index: integer): integer;
var
  i: integer;
begin
  Result:= -1;

  for i:= 0 to Count - 1 do
    if Items[i].GlyphIndex = Index then
    begin
      Result:= i;
      Exit;
    end;
end;

procedure THDSGlyphs.ClearIndexes;
var
  i: integer;
begin
  for i:= 0 to Count - 1 do
    Items[i].GlyphIndex:= -1;
end;

//THDSCustomGlyphGrid
constructor THDSCustomGlyphGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FGlyphMargins:= CreateMargins;
  FGlyphBorder:= bsSingle;
  FPicture:= CreatePicture;
  FLastGlyphs:= CreateLastGlyphs;
  FCurrentGlyph:= -1;
  FLastCol:= -1;
  FLastRow:= -1;
  FTitle:= '';
  FixedCols:= 0;
  FixedRows:= 0;
  RowCount:= 1;
  ColCount:= 1;
  FGlyphHeight:= 64;
  FGlyphWidth:= 64;
  FEllipseStyle:= ellEnd;
  FIsReading:= False;
  DefaultColWidth:= 64;
  DefaultRowHeight:= 64;
  GridLineWidth:= 0;
  FGlyphTextLayout:= gtlTop;
  FShowTitles:= False;
  FTextHeight:= 0;
  Options:= [];
end;

destructor THDSCustomGlyphGrid.Destroy;
begin
  FGlyphMargins.Free;
  FGlyphMargins:= nil;
  FPicture.Free;
  FPicture:= nil;
  FLastGlyphs.Free;
  FLastGlyphs:= nil;
  inherited Destroy;
end;

//Intercambio de propiedades
function THDSCustomGlyphGrid.GetColGlyphCount: integer;
begin
  Result:= inherited ColCount;
end;

function THDSCustomGlyphGrid.GetRowGlyphCount: integer;
begin
  Result:= inherited RowCount;
end;

procedure THDSCustomGlyphGrid.SetEllipseStyle(const Value: THDSEllipseStyle);
begin
  if FEllipseStyle <> Value then
  begin
    FEllipseStyle:= Value;
    Invalidate;
  end;
end;

procedure THDSCustomGlyphGrid.SetGlyphBorder(const Value: TBorderStyle);
begin
  if FGlyphBorder <> Value then
  begin
    FGlyphBorder:= Value;
    Invalidate;
  end;
end;

procedure THDSCustomGlyphGrid.SetGlyphHeight(const Value: integer);
begin
  if FGlyphHeight <> Value then
  begin
    FGlyphHeight:= Value;
    FLastGlyphs.ClearIndexes;
    DefaultRowHeight:= FGlyphHeight + FTextHeight;
  end;
end;

procedure THDSCustomGlyphGrid.SetGlyphTextLayout
                             (const Value: THDSGlyphTextLayout);
begin
  if FGlyphTextLayout <> Value then
  begin
    FGlyphTextLayout:= Value;

    if FShowTitles then
      Invalidate;
  end;
end;

procedure THDSCustomGlyphGrid.SetGlyphWidth(const Value: integer);
var
  AColCount: longint;
begin
  if FGlyphWidth <> Value then
  begin
    FGlyphWidth:= Value;
    FLastGlyphs.ClearIndexes;
    DefaultColWidth:= FGlyphWidth;
    AColCount:= Floor(ClientWidth / FGlyphWidth);

    if (AColCount > 0) and (ColCount <> AColCount) then
    begin
      ColCount:= AColCount;
      RowCount:= Ceil(ItemsCount / ColCount);
    end;
  end;
end;

function THDSCustomGlyphGrid.GetSelectedIndex: integer;
begin
  Result:= GridCoordToIndex(Col, Row);
end;

procedure THDSCustomGlyphGrid.SetSelectedIndex(const Value: integer);
var
  GridCoord: TGridCoord;
begin
  if (Value >= 0) and (Value < FItemsCount) then
  begin
    GridCoord:= IndexToGridCoord(Value);
    Col:= GridCoord.X;
    Row:= GridCoord.Y;
  end;
end;

procedure THDSCustomGlyphGrid.SetShowTitles(const Value: boolean);
begin
  if FShowTitles <> Value then
  begin
    FShowTitles:= Value;

    if FShowTitles then
    begin
      FTextHeight:= Canvas.TextHeight('Qq') + 5;
      DefaultRowHeight:= FGlyphHeight + FTextHeight;
    end
    else
      DefaultRowHeight:= FGlyphHeight;
  end;
end;

//Mensajes
procedure THDSCustomGlyphGrid.CMFontChanged(var Message: TMessage);
var
  ATextHeight: integer;
begin
  inherited;
  ATextHeight:= Canvas.TextHeight('Qq') + 5;

  if FShowTitles and (FTextHeight <> ATextHeight) then
  begin
    FTextHeight:= ATextHeight;
    DefaultRowHeight:= FGlyphHeight + FTextHeight;
  end;
end;

procedure THDSCustomGlyphGrid.WMSize(var Message: TWMSize);
var
  AChanged: boolean;
begin
  AChanged:= False;

  if Width <> Message.Width then
    AChanged:= True;

  inherited;

  if AChanged then
  begin
    inherited ColCount:= Floor(ClientWidth / FGlyphWidth);

    if ColCount > 0 then
      inherited RowCount:= Ceil(ItemsCount / ColCount);
  end;
end;

//Procedimientos
procedure THDSCustomGlyphGrid.Click;
begin
  with GetCurrentCell do
    if (X >= 0) and (X < ColCount) and (Y >= 0) and (Y < RowCount) then
      GlyphClick(X, Y, GetTitle(X, Y));
end;

procedure THDSCustomGlyphGrid.DblClick;
begin
  with GetCurrentCell do
    if (X >= 0) and (X < ColCount) and (Y >= 0) and (Y < RowCount) then
      GlyphDblClick(X, Y, GetTitle(X, Y));
end;

function THDSCustomGlyphGrid.CreatePicture: TPicture;
begin
  Result:= TPicture.Create;
end;

function THDSCustomGlyphGrid.CreateMargins: TMargins;
begin
  Result:= TMargins.Create(Self);
  Result.OnChange:= DoMarginsChanged;
end;

function THDSCustomGlyphGrid.CreateLastGlyphs: THDSGlyphs;
var
  i: integer;
begin
  Result:= THDSGlyphs.Create(Self, THDSGlyph);

  for i:= 0 to 31 do
    Result.Add;
end;

procedure THDSCustomGlyphGrid.DoMarginsChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure THDSCustomGlyphGrid.DrawCell(ACol, ARow: Longint; ARect: TRect;
                                       AState: TGridDrawState);
  var
    GridIndex, AWidth, AHeight: integer;

  function NextGlyph: integer;
  begin
    case FCurrentGlyph of
      -1, 31: FCurrentGlyph:= 0;
      else
        Inc(FCurrentGlyph);
    end;

    Result:= FCurrentGlyph;
  end;

  function DrawGlyph: integer;
  var
    AAHeight: integer;
  begin
    Result:= NextGlyph;
    AAHeight:= AHeight;

    if Picture.Width > AWidth then
      AHeight:= Floor(Picture.Height * (AWidth / Picture.Width));

    if Picture.Height > AHeight then
      AWidth:= Floor(Picture.Width * (AHeight / Picture.Height));

    if AHeight > AAHeight then
    begin
      AWidth:= Floor(AWidth * (AAHeight / AHeight));
      AHeight:= AAHeight;
    end;

    FLastGlyphs[Result].GlyphIndex:= GridIndex;

    with FLastGlyphs[Result].Bitmap do
    begin
      Width:= AWidth;
      Height:= AHeight;
      Canvas.StretchDraw(Rect(0, 0, Width, Height) , Picture.Graphic);
    end;
  end;

var
  AIndex: integer;
  TextRect: TRect;
  OldColor: TColor;
  IsSelected: boolean;
  Flags: integer;

const
  TEXT_FLAGS = DT_CENTER or DT_VCENTER or  DT_SINGLELINE or
               DT_NOPREFIX;
begin
  if GridCoordToIndex(ACol, ARow) = -1 then
  begin
    Canvas.Brush.Color:= Color;
    Canvas.FillRect(ARect);
  end
  else
  begin
    IsSelected:= gdSelected in AState;

    if IsSelected then
      Canvas.Brush.Color:= clHighlight
    else
      Canvas.Brush.Color:= Color;

    Canvas.FillRect(ARect);

    if FGlyphBorder = bsSingle then
      Frame3D(Canvas, ARect, clBtnHighlight, clBtnShadow, 2);

    if not FIsReading then
    begin
      FGlyphMargins.AlignRect(ARect);

      if FShowTitles and UpdateTitle(ACol, ARow) then
      begin
        case FEllipseStyle of
          ellEnd: Flags:= DT_END_ELLIPSIS;
          ellPath: Flags:= DT_PATH_ELLIPSIS;
          else //ellNone
            Flags:= 0;
        end;

        if FGlyphTextLayout = gtlTop then
        begin
          TextRect:= ARect;
          TextRect.Bottom:= TextRect.Top + FTextHeight;
          ARect.Top:= TextRect.Bottom + 1;
        end
        else
        begin
          TextRect:= ARect;
          TextRect.Top:= TextRect.Bottom - FTextHeight;
          ARect.Bottom:= TextRect.Top - 1;
        end;

        if IsSelected then
        begin
          OldColor:= Canvas.Font.Color;
          Canvas.Font.Color:= clHighlightText;
          DrawTextEx(Canvas.Handle, PChar(FTitle), -1, TextRect,
                     Flags or TEXT_FLAGS, nil);
          Canvas.Font.Color:= OldColor;
        end
        else
          DrawTextEx(Canvas.Handle, PChar(FTitle), -1, TextRect,
                     Flags or TEXT_FLAGS, nil);
      end;

      if UpdateImage(ACol, ARow) then
      begin
        with ARect do
        begin
          AWidth:= Right - Left;
          AHeight:= Bottom - Top;

          if (Picture.Width <= AWidth) and (Picture.Height <= AHeight) then
            Canvas.Draw(Left + ((AWidth - Picture.Width) div 2),
                        Top + ((AHeight - Picture.Height) div 2),
                        Picture.Graphic)
          else
          begin
            AIndex:= -1;
            GridIndex:= GridCoordToIndex(ACol, ARow);

            if FCurrentGlyph <> -1 then
              AIndex:= FLastGlyphs.FindIndex(GridIndex);

            if AIndex = -1 then
              AIndex:= DrawGlyph;

            CenterRect(ARect, FLastGlyphs[AIndex].Bitmap.Width,
                       FLastGlyphs[AIndex].Bitmap.Height);

            Canvas.StretchDraw(ARect, FLastGlyphs[AIndex].Bitmap);
          end;
        end;
      end;
    end;

    if Assigned(FOnDrawCell) then
      FOnDrawCell(Self, ACol, ARow, ARect, AState);
  end;
end;

function THDSCustomGlyphGrid.GetCurrentCell: TGridCoord;
var
  APoint: TPoint;
begin
  APoint:= ScreenToClient(Mouse.CursorPos);
  Result:= MouseCoord(APoint.x, APoint.y);
end;

procedure THDSCustomGlyphGrid.GlyphClick(ACol, ARow: longint;
                                         const ATitle: string);
begin
  if Assigned(FOnGlyphClick) then
    FOnGlyphClick(Self, GridCoordToIndex(ACol, ARow), ATitle);
end;

procedure THDSCustomGlyphGrid.GlyphDblClick(ACol, ARow: longint;
                                            const ATitle: string);
begin
  if Assigned(FOnGlyphDblClick) then
    FOnGlyphDblClick(Self, GridCoordToIndex(ACol, ARow), ATitle);
end;

function THDSCustomGlyphGrid.GridCoordToIndex(ACol, ARow: longint): integer;
begin
  Result:= (ARow * ColCount) + ACol;

  if (Result <  0) or (Result >= FItemsCount) then
    Result:= -1;
end;

function THDSCustomGlyphGrid.GridCoordToIndex(GridCoord: TGridCoord): integer;
begin
  Result:= GridCoordToIndex(GridCoord.X, GridCoord.Y);
end;

function THDSCustomGlyphGrid.IndexToGridCoord(AIndex: integer): TGridCoord;
begin
  with Result do
    if (AIndex >= 0) and (AIndex < FItemsCount) then
    begin
      Y:= AIndex div ColCount;
      X:= AIndex - (Y * ColCount);
    end
    else
    begin
      X:= -1;
      Y:= -1;
    end;
end;

function THDSCustomGlyphGrid.SelectCell(ACol, ARow: Longint): boolean;
begin
  if GridCoordToIndex(ACol, ARow) = -1 then
     Result:= False
  else
    Result := True;

  UpdateHint(ACol, ARow);
  
  if Result and Assigned(FOnSelectCell) then
    FOnSelectCell(Self, ACol, ARow, Result);
end;

procedure THDSCustomGlyphGrid.WndProc(var Message: TMessage);
var
  GridCoord: TGridCoord;
  APoint: TPoint;
begin
  if Message.Msg = WM_MOUSEMOVE then
  begin
    APoint:= ScreenToClient(Mouse.CursorPos);
    GridCoord:= MouseCoord(APoint.x, APoint.y);

    with GridCoord do
      if (X <> FLastCol) or (Y <> FLastRow) then
      begin
        FLastCol:= X;
        FLastRow:= Y;

        if UpdateHint(X, Y) then
          Application .HintMouseMessage(Self, Message);
      end;
  end;

  inherited WndProc(Message);
end;

//THDSGlyphGrid
constructor THDSGlyphGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFileList:= TStringList.Create;
  FSupportedExts:= GetSupportedExts;
end;

destructor THDSGlyphGrid.Destroy;
begin
  FFileList.Free;
  FFileList:= nil;
  inherited Destroy;
end;

//Intercambio de propiedades
procedure THDSGlyphGrid.SetDirectory(const Value: string);
begin
  if FDirectory <> Value then
  begin
    FDirectory:= Value;
    RefreshDirectory; 
    Invalidate;
  end;
end;

procedure THDSGlyphGrid.SetFilter(const Value: string);
begin
  if FFilter <> Value then
  begin
    FFilter:= Value;
    UpdateFiles;
    RowCount:= Ceil(ItemsCount / ColCount);
    Invalidate;
  end;
end;

//Porcedimientos
function THDSGlyphGrid.GetSupportedExts: string;
var
  APos: integer;
begin
  Result:= GraphicFilter(TGraphic);
  APos:= Pos('(', Result);

  if APos > 0 then
  begin
    Result:= Copy(Result, APos + 1, Length(Result) - APos - 1);
    APos:= Pos(')', Result);

    if APos > 0 then
      Result:= AnsiLowerCase(Copy(Result, 1, APos - 1))
    else
      Result:= '';
  end
  else
    Result:= '';
end;

function THDSGlyphGrid.GetTitle(ACol, ARow: longint): string;
var
  Index: longint;
begin
  Index:= (ARow * ColCount) + ACol;

  if (Index >= 0) and (Index < ItemsCount) then
    Result:= FFileList[Index]
  else
    Result:= '';
end;

function THDSGlyphGrid.UpdateFiles: boolean;
var
  AFilter, ADirectory, Extension: string;
  SearchRec: TSearchRec;
  APos, AResult: integer;


  function FindFirstGraphic(const AExtension: string): integer;
  begin
    Result:= FindFirst(ADirectory + AExtension, faArchive, SearchRec);
  end;
begin
  Result:= False;
  FFileList.Clear;

  if (FFilter <> '') and DirectoryExists(FDirectory) then
  begin
    ADirectory:= FDirectory;

    if ADirectory[Length(ADirectory)] <> '\' then
      ADirectory:= ADirectory + '\';

    AFilter:= FFilter;

    if AFilter[Length(AFilter)] <> ';' then
      AFilter:= AFilter + ';';

    APos:= Pos(';', AFilter);

    while APos > 0 do
    begin
      Extension:= Copy(AFilter, 1, APos - 1);

      if FindFirstGraphic(Trim(Extension)) = 0 then
      begin
        FFileList.Add(ADirectory + SearchRec.Name);

        repeat
          AResult:= FindNext(SearchRec);

          if AResult = 0 then
            FFileList.Add(ADirectory + SearchRec.Name);

        until AResult <> 0;

        FindClose(SearchRec);
      end;

      AFilter:= Copy(AFilter, APos + 1, Length(AFilter));
      APos:= Pos(';', AFilter);
    end;

    FFileList.Sorted:= True;
    FFileList.Sorted:= False; //Permite Modificar Items
    Result:= True;
  end;

  LastGlyphs.ClearIndexes;
  ItemsCount:= FFileList.Count;
end;

function THDSGlyphGrid.UpdateHint(ACol, ARow: integer): boolean;
var
  ATitle: string;
begin
  ATitle:= GetTitle(ACol, ARow);
  Hint:= ExtractFileName(ATitle) + '|' + ATitle ;
  Result:= ATitle <> '';
end;

function THDSGlyphGrid.UpdateImage(ACol, ARow: integer): boolean;
var
  Index: integer;
  HIcon: THandle;
  AIcon: Word;
begin
  Result:= False;
  Index:= (ARow * ColCount) + ACol;

  if (Index < ItemsCount) and FileExists(FFileList[Index]) then
  begin
    if Pos(AnsiLowerCase(ExtractFileExt(FFileList[Index])),
           FSupportedExts) > 0 then
      try
        Result:= True;
        Picture.LoadFromFile(FFileList[Index]);
      except
        Result:= False;
        FFileList[Index]:= 'None';
      end
    else
    begin
      HIcon:= ExtractAssociatedIcon(Application.Handle,
                                    PChar(FFileList[Index]), AIcon);

      if HIcon <> 0 then
      begin
        Result:= True;
        Picture.Icon.Handle:= HIcon;
      end
      else
        Result:= False;
    end;
  end;
end;

function THDSGlyphGrid.UpdateTitle(ACol, ARow: integer): boolean;
var
  Index: integer;
begin
  Result:= False;
  Index:= (ARow * ColCount) + ACol;

  if (Index < ItemsCount) and FileExists(FFileList[Index]) then
  begin
    Title:= ExtractFileName(FFileList[Index]);
    Result:= True;
  end;
end;

procedure THDSGlyphGrid.RefreshDirectory;
begin
  if UpdateFiles then
  begin
    FIsReading:= True;
    RowCount:= Ceil(ItemsCount / ColCount);
    FIsReading:= False;
    Invalidate;
  end
  else
  begin
    RowCount:= 1;
    ColCount:= 1;
  end;
end;
  
//THDSEllipseGrid
constructor THDSEllipseGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Color:= clBtnFace;
  ColCount:= 1;
  GridLineWidth:= 0;
  FAlignment:= taLeftJustify;
  FBevelWidth:= 1;
  FEllipseStyle:= ellEnd;
end;

procedure THDSEllipseGrid.SetAlignment(const Value: TAlignment);
begin
   if FAlignment <> Value then
   begin
     FAlignment:= Value;
     Invalidate;
   end;
end;

procedure THDSEllipseGrid.SetBevelWidth(const Value: TBevelWidth);
begin
   if FBevelWidth <> Value then
   begin
     FBevelWidth:= Value;
     Invalidate;
   end;
end;

procedure THDSEllipseGrid.SetEllipseStyle(const Value: THDSEllipseStyle);
begin
  if FEllipseStyle <> Value then
  begin
    FEllipseStyle:= Value;
    Invalidate;
  end;
end;

procedure THDSEllipseGrid.DrawCell(ACol, ARow: longint; ARect: TRect;
                                   AState : TGridDrawState);

  procedure ResizeRect(Value: integer);
  begin
    with ARect do
    begin
      Inc(Left, Value);
      Dec(Right, Value);
      Inc(Top, Value);
      Dec(Bottom, Value);
    end;
  end;
var
   AText: string;
   IsSelected: boolean;
   Flags: longint;
   OldColor: TColor;

const
    TEXT_FLAGS = DT_VCENTER or  DT_SINGLELINE or DT_NOPREFIX;
begin
   IsSelected:= gdSelected in AState;
   Canvas.Brush.Color:= Color;
   Canvas.FillRect(ARect);
   ResizeRect(1);

   if IsSelected then
   begin
      Frame3D(Canvas, ARect, clBtnShadow, clBtnHighlight, FBevelWidth);
      Canvas.Brush.Color:= clHighlight;
      Canvas.FillRect(ARect);
   end
   else
   begin
     Frame3D(Canvas, ARect, clBtnHighlight, clBtnShadow, FBevelWidth);

     if gdFixed in AState  then
     begin
        Canvas.Brush.Color:= FixedColor;
        Canvas.FillRect(ARect);
     end;
   end;

   ResizeRect(2);
   AText:= Cells[ACol, ARow];

   case FEllipseStyle of
      ellPath: Flags:= DT_PATH_ELLIPSIS;
      ellEnd: Flags:= DT_END_ELLIPSIS;
      else //ellNone
        Flags:= 0;
   end;

   case FAlignment of
      taLeftJustify: Flags:= Flags or DT_LEFT ;
      taCenter: Flags:= Flags or DT_CENTER;
      else //taRightJustify
        Flags:= Flags or DT_RIGHT;
   end;

   if IsSelected then
   begin
      OldColor:= Canvas.Font.Color;
      Canvas.Font.Color:= clHighlightText;
      DrawTextEx(Canvas.Handle, PChar(AText), -1, ARect,
                 TEXT_FLAGS or Flags, nil);
      Canvas.Font.Color:= OldColor;
   end
   else
      DrawTextEx(Canvas.Handle, PChar(AText), -1, ARect,
                 TEXT_FLAGS or Flags, nil);

   if Assigned(OnDrawCell) then
      OnDrawCell(Self, ACol, ARow, ARect, AState);
end;

procedure Register;
begin
  RegisterComponents('Hades', [THDSGlyphGrid, THDSEllipseGrid]);
end;

end.
